'use client';

import React, { useEffect, useState } from 'react';
import ApiClient from '../services/ApiClient';

interface PushNotificationHandlerProps {
    onSubscriptionChange?: (subscription: PushSubscription | null) => void;
}

const PushNotificationHandler: React.FC<PushNotificationHandlerProps> = () => {
    const [swRegistration, setSwRegistration] = useState<ServiceWorkerRegistration | null>(null);
    const [subscription, setSubscription] = useState<PushSubscription | null>(null);

    useEffect(() => {
        if (typeof window !== 'undefined' && 'serviceWorker' in navigator && 'PushManager' in window && process.env.NODE_ENV === 'production') {
            navigator.serviceWorker.register(`${process.env.ASSET_PREFIX}/sw.js`)
                .then(registration => {
                    setSwRegistration(registration);
                    initializePush(registration);
                })
                .catch(error => {
                    console.error('Service Worker registration failed:', error);
                });
        } else {
            console.log('Push notifications are not supported');
        }
    }, []);

    const initializePush = async (registration: ServiceWorkerRegistration) => {
        if (Notification.permission === 'granted') {
            const sub = await subscribeUser(registration);
            setSubscription(sub);
        } else if (Notification.permission !== 'denied') {
            const permission = await Notification.requestPermission();
            if (permission === 'granted') {
                const sub = await subscribeUser(registration);
                setSubscription(sub);
            }
        }
    };

    const subscribeUser = async (registration: ServiceWorkerRegistration): Promise<PushSubscription | null> => {
        try {
            let sub = await registration.pushManager.getSubscription();
            if (sub) {
                const existingKey = sub.options.applicationServerKey;
                const newKey = urlBase64ToUint8Array(process.env.VAPID_PUBLIC_KEY!);

                if (existingKey && !arraysEqual(new Uint8Array(existingKey), newKey)) {
                    await sub.unsubscribe();
                    sub = await registration.pushManager.subscribe({
                        userVisibleOnly: true,
                        applicationServerKey: newKey
                    });
                }
            } else {
                sub = await registration.pushManager.subscribe({
                    userVisibleOnly: true,
                    applicationServerKey: urlBase64ToUint8Array(process.env.VAPID_PUBLIC_KEY!)
                });
            }

            await storePushSubscription(sub);
            return sub;
        } catch (error) {
            console.error('Failed to subscribe the user:', error);
            return null;
        }
    };

    const storePushSubscription = async (pushSubscription: PushSubscription): Promise<void> => {
        try {
            const response = await ApiClient.post('chat/push', pushSubscription);
            const data = await response.data;
            console.log('Push subscription stored on server', data);
        } catch (error) {
            console.error('Error storing push subscription:', error);
        }
    };

    const urlBase64ToUint8Array = (base64String: string): Uint8Array => {
        const padding = '='.repeat((4 - base64String.length % 4) % 4);
        const base64 = (base64String + padding)
            .replace(/-/g, '+')
            .replace(/_/g, '/');

        const rawData = window.atob(base64);
        const outputArray = new Uint8Array(rawData.length);

        for (let i = 0; i < rawData.length; ++i) {
            outputArray[i] = rawData.charCodeAt(i);
        }
        return outputArray;
    };

    const arraysEqual = (a: Uint8Array, b: Uint8Array): boolean => {
        if (a.length !== b.length) return false;
        return a.every((val, index) => val === b[index]);
    };

    return null; // Este componente no renderiza nada visible
};

export default PushNotificationHandler;